

function [alldata] =uXl2ml(sFile, sSheetname)
%function [alldata] =uXl2ml(sFile, sSheetname)
%
% This function reads columnwise data  from an Excel sheet into ECOMT-STRUCTURES
%   Ecomt-Structure: object.data=double column vector, object.name = string
%
% There may or be may not be data headers (this function considers as headers
%   only the first row containing strings) or they may exist partially: This
%   function provides a unique name for each series and saves it into the
%   base series
%
% Caution: This function always assumes the first column to contain
%   observations indices
%____________________________________________________
%OUTPUTS
%   alldata: array of Ecomt structures
%____________________________________________________
%INPUTS
%   sFile (optional): string File name; if not given or not existing, the function
%       asks for it again
%   sSheetname (optional): name or integer number of the sheet to be read
%       in (default=first sheet). If your Matlab is below version number 7,
%       then always the first Sheet is taken
%____________________________________________________
%EXAMPLE
% fncXl2ml('asparagus.xls')
%____________________________________________________
% This function was entirely re-done based on the work by Dominik
% Grafenhofer
% main ideas taken from Dominik: assignin, structures
%______________________________________________________________________
%   Martin Feldkircher & Stefan Zeugner 2006

lVersion=str2num(strtok(version,'.'));

%if no file name provided, ask for it
if nargin == 0 | exist(sFile)~=2
    for i=1:5
        sFile=input('Provide file name:   ', 'S');
        if exist(sFile)==2
            break
        end
        if i==5
            %if user doesn't know 5 times, exit
            error('too many trys: exiting ...')
        end
    end
end

if nargin <2, sSheetname=1; end %take first worksheet if not specified

if lVersion<7
    [dData, sHeaders]=xlsread(sFile); %read in data
else
    [dData, sHeaders]=xlsread(sFile,sSheetname); %read in data
end
switch size(sHeaders,2)-size(dData, 2)

    case 0
    %obs are doubles
        obs.data=dData(:,1);
        obs.name=char(sHeaders(1,1));
        sHeaders=sHeaders(1,2:size(sHeaders,2));
        dData=dData(:,2:size(dData,2));
    case 1
    %obs are strings
        obs.data=sHeaders(2:size(sHeaders,1),1);
        obs.name=sHeaders(1,1);
        sHeaders=sHeaders(1,2:size(sHeaders,2));

    otherwise
       if size(sHeaders,1)==size(dData, 1)
            %no data headers and obs are strings
            obs.data=sHeaders;
            obs.name='OBS';
            sHeaders=sHeaders(1,1);
            for i=1:size(dData,2)
                sHeaders(1,i)={strcat('ser',int2str(i))}; % name data headers ser1,. ser2, etc.
            end
       elseif size(sHeaders,1)==0
           %no data headers and obs are doubles
            obs.data=dData(:,1);
            obs.name='OBS';
            dData=dData(:,2:size(dData,2));
            for i=1:size(dData,2) % name data headers ser1,. ser2, etc.
                sHeaders(1,i)={strcat('ser',int2str(i))};
            end
       else
           %Problem
           error('Error reading in data..., data and headers could not be identified')
       end

end

    if isempty(char(obs.name)), obs.name='OBS'; end %name obs if they aren't
    if lVersion<7
        sobsvar=obs.name;
    else
        sobsvar=genvarname(char(obs.name),evalin('base','who'));
    end
    if lVersion<7;
        assignin('base', sobsvar{1,1}, obs); %assign obs to base workspace
    else
        assignin('base', sobsvar, obs); %assign obs to base workspace
    end

for i=1:size(dData,2)
   %same structures in order to ensure compatibility with Dominik
   
   if isempty(char(sHeaders(1,i)))
       sHeaders(1,i)={strcat('ser',int2str(i))}; %if no name then call it 'ser' + column number
   end
   alldata(i).name = char(sHeaders(1,i));
   alldata(i).data = dData(:,i);
    if lVersion<7
        sservar=char(sHeaders(1,i));
    else
        sservar=genvarname(char(sHeaders(1,i)),evalin('base','who'));
    end
   assignin('base', sservar, alldata(i)); %assign the thing into base workspace

end

disp(strcat('Reading-in completed: ',int2str(i),' columns and the observations been read in.'))

%   stz 2006